/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var Room = (function () {
    function Room(ws, fullName) {
        this.eventHandlers = {};
        this.ws = ws;
        var _a = fullName.split(':'), topic = _a[0], room = _a[1];
        this.topic = topic;
        this.room = room;
    }
    Room.prototype.join = function (opts) {
        if (this.isJoined) {
            throw Room.errorAlreadyJoined;
        }
        this.ws.send(JSON.stringify({
            topic: this.topic,
            room: this.room,
            event: 'join',
            payload: opts,
        }));
    };
    Room.prototype.send = function (event, payload) {
        if (!this.isJoined) {
            throw Room.errorNotJoined;
        }
        this.ws.send(JSON.stringify({
            event: event,
            payload: payload,
            topic: this.topic,
            room: this.room,
        }));
    };
    Room.prototype.leave = function (payload) {
        if (!this.isJoined) {
            return;
        }
        this.ws.send(JSON.stringify({
            payload: payload,
            topic: this.topic,
            room: this.room,
            event: 'leave',
        }));
    };
    Room.prototype.onEvent = function (event, handler) {
        if (!this.eventHandlers[event]) {
            this.eventHandlers[event] = [];
        }
        this.eventHandlers[event].push(handler);
    };
    Room.prototype.messageReceived = function (event, payload) {
        switch (event) {
            case 'joined':
                this.isJoined = true;
                break;
            case 'left':
                this.isJoined = false;
                break;
            default:
                break;
        }
        if (this.eventHandlers[event]) {
            this.eventHandlers[event].forEach(function (h) { return h(event, payload); });
        }
        else {
            console.warn("Event (" + event + ") not being watched");
        }
    };
    Room.errorAlreadyJoined = new Error('Already Joined');
    Room.errorNotJoined = new Error('Not Joined');
    return Room;
}());

var Socket = (function () {
    function Socket(url, opts) {
        this.opts = {
            timeout: 1000,
            backoffScale: 1.5,
        };
        this.currentTimeout = 0;
        this.openHandlers = [];
        this.closeHandlers = [];
        this.errorHandlers = [];
        this.rooms = {};
        this.url = url;
        this.opts = __assign({}, this.opts, opts);
        this.currentTimeout = this.opts.timeout;
        this.connect = this.connect.bind(this);
        this.onOpen = this.onOpen.bind(this);
        this.onClose = this.onClose.bind(this);
        this.onError = this.onError.bind(this);
    }
    Socket.prototype.connect = function () {
        var _this = this;
        this.ws = new WebSocket(this.url);
        this.ws.addEventListener('open', function (e) {
            console.info('connected to socket at:', _this.url);
            _this.currentTimeout = _this.opts.timeout;
            _this.openHandlers.forEach(function (fn) { return fn(e); });
        });
        this.ws.addEventListener('close', function (e) {
            _this.ws = null;
            if (!e.wasClean) {
                console.error('Socket closed:', e);
                setTimeout(_this.connect, _this.currentTimeout);
                _this.currentTimeout = _this.currentTimeout * _this.opts.backoffScale;
            }
            _this.closeHandlers.forEach(function (fn) { return fn(e); });
        });
        this.ws.addEventListener('error', function (e) {
            console.error('Socket error:', e);
            _this.errorHandlers.forEach(function (fn) { return fn(e); });
        });
        this.ws.addEventListener('message', function (e) {
            var msg = JSON.parse(e.data);
            var channel = msg.topic + ":" + msg.room;
            if (_this.rooms[channel]) {
                _this.rooms[channel].messageReceived(msg.event, msg.payload);
            }
            else {
                console.warn("Channel (" + channel + ") not found for message:", msg);
            }
        });
    };
    Socket.prototype.join = function (channel, opts) {
        if (!this.ws) {
            throw Socket.notConnectedError;
        }
        var room = new Room(this.ws, channel);
        this.rooms[channel] = room;
        room.join(opts);
        return room;
    };
    Socket.prototype.onOpen = function (fn) {
        this.openHandlers.push(fn);
    };
    Socket.prototype.onClose = function (fn) {
        this.closeHandlers.push(fn);
    };
    Socket.prototype.onError = function (fn) {
        this.errorHandlers.push(fn);
    };
    Socket.notConnectedError = new Error('Not connected to WebSocket');
    return Socket;
}());

export { Room, Socket };
